<?php

namespace Drupal\ts_admin_base_theme\Preprocess\Paragraphs;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\ts_dx\Services\Theme\TwigExtension;

/**
 * Preprocess for paragraphs.
 *
 * @package Drupal\ts_admin_base_theme\Preprocess\Paragraphs
 */
class ParagraphsPreprocess {

  /**
   * Singleton.
   *
   * @var static
   */
  protected static $me;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Theme tools.
   *
   * @var \Drupal\ts_dx\Services\Theme\TwigExtension
   */
  protected $twigExtension;

  /**
   * Cache des types de paragraphs par label.
   *
   * @var \Drupal\paragraphs\Entity\ParagraphsType[]
   */
  protected $paragraphsLabelCache = [];

  /**
   * ParagraphsPreprocess constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\ts_dx\Services\Theme\TwigExtension $twigExtension
   *   The twig extension.
   */
  private function __construct(EntityTypeManagerInterface $entityTypeManager, TwigExtension $twigExtension) {
    $this->entityTypeManager = $entityTypeManager;
    $this->twigExtension = $twigExtension;
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    if (!isset(static::$me)) {
      static::$me = new static(
        \Drupal::service('entity_type.manager'),
        TwigExtension::me()
      );
    }

    return static::$me;
  }

  /**
   * Initialise le tableau de paragraphe côté edition d'entité.
   *
   * @param array $variables
   *   Les variables preprocess.
   */
  public function initParagraphsListBuilder(array &$variables) {
    $keys = ['element', 'label', '#markup'];

    // Récuépration du label du paragraphe en train d'être printé.
    if ($label = NestedArray::getValue($variables, $keys)) {
      $label = trim(strip_tags($label));

      // Si on a trouvé le parargaphe type, ajoute les contenus image et titre.
      if ($paragraphType = $this->getParagraphTypeFromParagraphLabel($label)) {
        $variables['attributes']['data-bundle'] = $paragraphType->id();
        $variables['attributes']['class'][] = 'paragraph-head-wrapper';

        $thumbnailPath = $this->getParagraphTypeThumbnailPath($paragraphType);

        $variables['children'] = [
          [
            '#type'       => 'container',
            '#attributes' => ['class' => ['img-wrapper']],
            0             => [
              '#markup' => $thumbnailPath ? '<img alt="" src="' . $this->twigExtension->getCurrentModulePath('assets/paragraphs/' . $paragraphType->id() . '.svg') . '">':'',
            ],
          ],
          [
            '#type'       => 'container',
            '#attributes' => ['class' => ['title']],
            0             => [
              '#markup' => $paragraphType->label(),
            ],
          ],
        ];
      }
    }
  }

  /**
   * Retourne le type de paragraphe en fonction du label.
   *
   * @param string $label
   *   Le label.
   *
   * @return \Drupal\paragraphs\Entity\ParagraphsType|null
   *   Le type de paragraph.
   */
  public function getParagraphTypeFromParagraphLabel(string $label) {
    if (!isset($this->paragraphsLabelCache[$label])) {
      if ($paragraphsTypesDefinition = $this->entityTypeManager->getDefinition('paragraphs_type')) {
        $storage = $this->entityTypeManager->getStorage('paragraphs_type');
        $labelField = $paragraphsTypesDefinition->getKey('label');
        $result = $storage->loadByProperties([
          $labelField => $label,
        ]);
        $this->paragraphsLabelCache[$label] = reset($result);
      }
      else {
        $this->paragraphsLabelCache[$label] = NULL;
      }
    }

    return $this->paragraphsLabelCache[$label];
  }

  /**
   * Initialise les libraries utilisées par la liste de type de paragraphe.
   *
   * @param array $variables
   *   Les variables de rendu.
   */
  public function initParagraphsList(array &$variables) {
    // Ajout de la libraries.
    $variables['#attached']['library'][] = 'ts_admin_base_theme/paragraphs';
    $variables['#attached']['drupalSettings']['path']['theme_path'] = $this->twigExtension->getCurrentModulePath('');

    $variables['page']['content']['#prefix'] = '<div class="paragraphs-types-table">';
    $variables['page']['content']['#suffix'] = '</div>';
  }

  /**
   * Récupération de l'image de thumbnail du paragraph
   *
   * @param \Drupal\paragraphs\ParagraphsTypeInterface $paragraphType
   */
  protected function getParagraphTypeThumbnailPath($paragraphType) {
    $path = null;
    $root = \Drupal::root();
    $pattern = $root . $this->twigExtension->getCurrentModulePath('assets/paragraphs/' . $paragraphType->id() . '.*');
    $files = glob($pattern);
    if( $firstFile = reset($files) ){
      $path = str_replace($root, '', $firstFile);
    }

    return $path;
  }

}
